@@ -20,27 +20,33 @@
   :group 'rustic-cargo)
 
 (defcustom rustic-cargo-check-exec-command "check"
-  "Execute command to run cargo check."
+  "Execute command to run `cargo check'."
   :type 'string
   :group 'rustic-cargo)
 
 (defcustom rustic-cargo-test-exec-command "test"
-  "Execute command to run cargo test."
+  "Execute command to run `cargo test'."
   :type 'string
   :group 'rustic-cargo)
 
-(defcustom rustic-cargo-nextest-exec-command "nextest run"
+(defcustom rustic-cargo-test-runner 'cargo
+  "Test runner to use for running tests.  By default uses cargo."
+  :type '(choice (const cargo)
+                 (const nextest))
+  :group 'rustic-cargo)
+
+(defcustom rustic-cargo-nextest-exec-command (list "nextest" "run")
   "Execute command to run nextest."
-  :type 'string
+  :type '(repeat string)
   :group 'rustic-cargo)
 
 (defcustom rustic-cargo-run-exec-command "run"
-  "Execute command to run cargo run."
+  "Execute command to run `cargo run'."
   :type 'string
   :group 'rustic-cargo)
 
 (defcustom rustic-cargo-build-exec-command "build"
-  "Execute command to run cargo build."
+  "Execute command to run `cargo build'."
   :type 'string
   :group 'rustic-cargo)
 
@@ -49,6 +55,40 @@
   :type 'string
   :group 'rustic-cargo)
 
+(defcustom rustic-cargo-populate-package-name nil
+  "Populate package name automatically when used with universal argument."
+  :type 'boolean
+  :group 'rustic-cargo)
+
+(defvar rustic--package-names (make-hash-table :test #'equal))
+
+(defun rustic-cargo-cached-package-name ()
+  (let ((package-name (gethash default-directory rustic--package-names)))
+    (if package-name
+        package-name
+      (progn
+        (let ((pkg-name (rustic-cargo-package-name)))
+          (setf (gethash default-directory rustic--package-names) pkg-name))
+        (gethash default-directory rustic--package-names)))))
+
+(defun rustic-cargo-package-argument ()
+  (if rustic-cargo-populate-package-name
+      (let ((package-name (rustic-cargo-cached-package-name)))
+        (when package-name
+          (format "--package %s" package-name)))))
+
+(defun rustic-cargo-package-name ()
+  (let ((buffer (get-buffer "*cargo-manifest*")))
+    (if buffer
+        (kill-buffer buffer)))
+  (let* ((buffer (get-buffer-create "*cargo-manifest*"))
+         (exit-code (call-process (rustic-cargo-bin) nil buffer nil "read-manifest")))
+    (if (eq exit-code 0)
+        (with-current-buffer buffer
+          (let ((json-parsed-data (json-read-from-string (buffer-string))))
+            (cdr (assoc 'name json-parsed-data))))
+      nil)))
+
 (defun rustic-cargo-bin ()
   (if (file-remote-p (or (buffer-file-name) ""))
       rustic-cargo-bin-remote
@@ -61,7 +101,7 @@ If nil then the project is simply created."
   :group 'rustic-cargo)
 
 (defcustom rustic-cargo-test-disable-warnings nil
-  "Don't show warnings when running 'cargo test'."
+  "Don't show warnings when running `cargo test'."
   :type 'boolean
   :group 'rustic-cargo)
 
@@ -77,30 +117,30 @@ instead of applying the default arguments from `rustic-default-test-arguments'."
   :type 'boolean
   :group 'rustic-cargo)
 
-(defcustom rustic-default-test-arguments "--benches --tests --all-features"
-  "Default arguments when running 'cargo test'."
+(defcustom rustic-default-test-arguments "--all-targets --all-features"
+  "Default arguments when running `cargo test'."
   :type 'string
   :group 'rustic-cargo)
 
 (defcustom rustic-cargo-default-install-arguments '("--path" "." "--locked")
-  "Default arguments when running 'cargo install'."
+  "Default arguments when running `cargo install'."
   :type '(list string)
   :group 'rustic-cargo)
 
-(defcustom rustic-cargo-check-arguments "--benches --tests --all-features"
-  "Default arguments when running 'cargo check'."
+(defcustom rustic-cargo-check-arguments "--all-targets --all-features"
+  "Default arguments when running `cargo check'."
   :type 'string
   :group 'rustic-cargo)
 
 (defcustom rustic-cargo-build-arguments ""
-  "Default arguments when running 'cargo build'."
+  "Default arguments when running `cargo build'."
   :type 'string
   :group 'rustic-cargo)
 
 (defcustom rustic-cargo-auto-add-missing-dependencies nil
   "Automatically adds dependencies to Cargo.toml.
-This way rustic checks new diagnostics for 'unresolved import'
-errors and passes the crates to 'cargo add'.
+This way rustic checks new diagnostics for `unresolved import'
+errors and passes the crates to `cargo add'.
 Currently only working with lsp-mode."
   :type 'boolean
   :group 'rustic-cargo)
@@ -132,14 +172,19 @@ Currently only working with lsp-mode."
   "Buffer name for test buffers.")
 
 (defvar rustic-test-arguments ""
-  "Holds arguments for 'cargo test', similar to `compilation-arguments`.
+  "Holds arguments for `cargo test', similar to `compilation-arguments`.
 Tests that are executed by `rustic-cargo-current-test' will also be
 stored in this variable.")
 
+(defvar rustic-test-history nil
+  "Holds previous arguments for `cargo test', similar to `compile-arguments`.")
+
 (defvar rustic-cargo-test-mode-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map rustic-compilation-mode-map)
     (define-key map [remap recompile] 'rustic-cargo-test-rerun)
+    (define-key map (kbd "C-c C-t") 'rustic-cargo-test-rerun-current)
+    (define-key map (kbd "t") 'rustic-cargo-test-rerun-current)
     map)
   "Local keymap for `rustic-cargo-test-mode' buffers.")
 
@@ -149,24 +194,18 @@ stored in this variable.")
   (when rustic-cargo-test-disable-warnings
     (setq-local rustic-compile-rustflags (concat rustic-compile-rustflags " -Awarnings"))))
 
-(defun rustic-cargo-run-nextest (&optional arg)
-  "Command for running nextest."
-  (interactive "P")
-  (let* ((nextest (if arg
-                      (read-from-minibuffer "nextest command: " rustic-cargo-nextest-exec-command)
-                    rustic-cargo-nextest-exec-command))
-         (c (-flatten (list (rustic-cargo-bin) (split-string nextest))))
-         (buf rustic-test-buffer-name)
-         (proc rustic-test-process-name)
-         (mode 'rustic-cargo-test-mode))
-    (rustic-compilation c (list :buffer buf :process proc :mode mode))))
+(defun rustic--cargo-test-runner ()
+  "Return the test runner command."
+  (cond ((eq rustic-cargo-test-runner 'cargo) rustic-cargo-test-exec-command)
+        ((eq rustic-cargo-test-runner 'nextest) rustic-cargo-nextest-exec-command)
+        (t (user-error "Invalid configured value for rustic-cargo-test-runner variable"))))
 
 ;;;###autoload
 (defun rustic-cargo-test-run (&optional test-args)
-  "Start compilation process for 'cargo test' with optional TEST-ARGS."
+  "Start compilation process for `cargo test' with optional TEST-ARGS."
   (interactive)
   (rustic-compilation-process-live)
-  (let* ((command (list (rustic-cargo-bin) rustic-cargo-test-exec-command))
+  (let* ((command (flatten-list (list (rustic-cargo-bin) (rustic--cargo-test-runner))))
          (c (append command (split-string (if test-args test-args ""))))
          (buf rustic-test-buffer-name)
          (proc rustic-test-process-name)
@@ -175,42 +214,77 @@ stored in this variable.")
 
 ;;;###autoload
 (defun rustic-cargo-test (&optional arg)
-  "Run 'cargo test'.
+  "Run `cargo test'.
 
 If ARG is not nil, use value as argument and store it in
 `rustic-test-arguments'.  When calling this function from
 `rustic-popup-mode', always use the value of
 `rustic-test-arguments'."
   (interactive "P")
-  (when arg
-    (setq rustic-test-arguments
-          (read-from-minibuffer "Cargo test arguments: "
-                                (rustic--populate-minibuffer
-                                 (list rustic-test-arguments
-                                       rustic-cargo-build-arguments
-                                       rustic-default-test-arguments)))))
-  (rustic-cargo-test-run rustic-test-arguments))
+  (rustic-cargo-test-run
+   (cond (arg
+          (setq rustic-test-arguments
+                (read-from-minibuffer "Cargo test arguments: "
+                                      (rustic--populate-minibuffer
+                                       (list (rustic-cargo-package-argument)
+                                             rustic-test-arguments
+                                             rustic-cargo-build-arguments
+                                             rustic-default-test-arguments))
+                                      nil nil 'rustic-test-history)))
+         (rustic-cargo-use-last-stored-arguments
+          (if (> (length rustic-test-arguments) 0)
+              rustic-test-arguments
+            rustic-default-test-arguments))
+         (t
+          rustic-default-test-arguments))))
 
 ;;;###autoload
-(defun rustic-cargo-test-rerun ()
-  "Run 'cargo test' with `rustic-test-arguments'."
-  (interactive)
+(defun rustic-cargo-test-rerun (arg)
+  "Run `cargo test' with `rustic-test-arguments'."
+  (interactive "P")
   (let ((default-directory (or rustic-compilation-directory default-directory)))
+    (setq rustic-test-arguments
+          (if arg
+              (read-from-minibuffer "Cargo test arguments: " rustic-test-arguments nil nil 'rustic-test-history)
+            rustic-test-arguments))
     (rustic-cargo-test-run rustic-test-arguments)))
 
+(defun rustic-cargo-test-rerun-current (arg)
+  "Rerun the test at point from `rustic-cargo-test-mode'."
+  (interactive "P")
+  (let* ((default-directory (or rustic-compilation-directory default-directory))
+        (test (rustic-cargo--get-test-at-point))
+        (command (if test
+                     (concat "-- --exact " test)
+                   (error "No test found at point"))))
+    (setq rustic-test-arguments
+          (if arg
+              (read-from-minibuffer "Cargo test arguments: " command nil nil 'rustic-test-history)
+            command))
+    (rustic-cargo-test-run rustic-test-arguments)))
+
+(defun rustic-cargo--get-test-at-point ()
+  (save-excursion
+      (beginning-of-line)
+      (when (re-search-forward "^test \\([^ ]+\\) ..." (line-end-position) t)
+        (buffer-substring-no-properties (match-beginning 1) (match-end 1)))))
+
 ;;;###autoload
 (defun rustic-cargo-current-test ()
-  "Run 'cargo test' for the test near point."
+  "Run `cargo test' for the test near point."
   (interactive)
   (rustic-compilation-process-live)
   (-if-let (test-to-run (setq rustic-test-arguments
                               (rustic-cargo--get-test-target)))
-      (rustic-cargo-run-test test-to-run)
+      (progn
+        (unless (equal (car rustic-test-history) test-to-run)
+          (push test-to-run rustic-test-history))
+        (rustic-cargo-run-test test-to-run))
     (message "Could not find test at point.")))
 
 (defun rustic-cargo-run-test (test)
   "Run TEST which can be a single test or mod name."
-  (let* ((c (list (rustic-cargo-bin) rustic-cargo-test-exec-command test))
+  (let* ((c (flatten-list (list (rustic-cargo-bin) (rustic--cargo-test-runner) test)))
          (buf rustic-test-buffer-name)
          (proc rustic-test-process-name)
          (mode 'rustic-cargo-test-mode))
@@ -303,7 +377,7 @@ If ARG is not nil, use value as argument and store it in
 
 ;;;###autoload
 (defun rustic-cargo-outdated (&optional path)
-  "Use 'cargo outdated' to list outdated packages in `tabulated-list-mode'.
+  "Use `cargo outdated' to list outdated packages in `tabulated-list-mode'.
 Execute process in PATH."
   (interactive)
   (rustic--inheritenv
@@ -540,7 +614,7 @@ exist, and the rest. Return a cons cell of the two parts."
     (cons existing-dir (apply 'file-name-concat (cons "" components)))))
 
 (defun rustic-create-project (project-path is-new &optional bin)
-  "Run either 'cargo new' if IS-NEW is non-nil, or 'cargo init' otherwise.
+  "Run either `cargo new' if IS-NEW is non-nil, or `cargo init' otherwise.
 Creates or initializes the directory at the path specified by PROJECT-PATH. If
 BIN is not nil, create a binary application, otherwise a library."
   (let* ((cmd (if is-new "new" "init"))
@@ -568,14 +642,14 @@ BIN is not nil, create a binary application, otherwise a library."
 
 ;;;###autoload
 (defun rustic-cargo-new (project-path &optional bin)
-  "Run 'cargo new' to start a new package in the path specified by PROJECT-PATH.
+  "Run `cargo new' to start a new package in the path specified by PROJECT-PATH.
 If BIN is not nil, create a binary application, otherwise a library."
   (interactive "GProject path: ")
   (rustic-create-project project-path t bin))
 
 ;;;###autoload
 (defun rustic-cargo-init (project-path &optional bin)
-  "Run 'cargo init' to initialize a directory in the path specified by PROJECT-PATH.
+  "Run `cargo init' to initialize a directory in the path specified by PROJECT-PATH.
 If BIN is not nil, create a binary application, otherwise a library."
   (interactive "DProject path: ")
   (rustic-create-project project-path nil bin))
@@ -589,7 +663,10 @@ If BIN is not nil, create a binary application, otherwise a library."
   "Buffer name for run buffers.")
 
 (defvar rustic-run-arguments ""
-  "Holds arguments for 'cargo run', similar to `compilation-arguments`.")
+  "Holds arguments for `cargo run', similar to `compilation-arguments`.")
+
+(defvar rustic-run-history nil
+  "Holds previous arguments for `cargo run', similar to `compile-history`.")
 
 (defvar rustic-cargo-run-mode-map
   (let ((map (make-sparse-keymap)))
@@ -602,7 +679,7 @@ If BIN is not nil, create a binary application, otherwise a library."
 
 ;;;###autoload
 (defun rustic-cargo-run-command (&optional run-args)
-  "Start compilation process for 'cargo run' with optional RUN-ARGS."
+  "Start compilation process for `cargo run' with optional RUN-ARGS."
   (interactive)
   (rustic-compilation-process-live)
   (let* ((command (list (rustic-cargo-bin) rustic-cargo-run-exec-command))
@@ -614,7 +691,7 @@ If BIN is not nil, create a binary application, otherwise a library."
 
 ;;;###autoload
 (defun rustic-cargo-run (&optional arg)
-  "Run 'cargo run'.
+  "Run `cargo run'.
 
 If ARG is not nil, use value as argument and store it in `rustic-run-arguments'.
 When calling this function from `rustic-popup-mode', always use the value of
@@ -622,27 +699,34 @@ When calling this function from `rustic-popup-mode', always use the value of
   (interactive "P")
   (rustic-cargo-run-command
    (cond (arg
-          (setq rustic-run-arguments (read-from-minibuffer "Cargo run arguments: " rustic-run-arguments)))
+          (setq rustic-run-arguments
+                (read-from-minibuffer "Cargo run arguments: "
+                                      rustic-run-arguments nil nil 'rustic-run-history)))
          (rustic-cargo-use-last-stored-arguments
           rustic-run-arguments)
          ((rustic--get-run-arguments))
          (t rustic-run-arguments))))
 
 ;;;###autoload
-(defun rustic-cargo-run-rerun ()
-  "Run 'cargo run' with `rustic-run-arguments'."
-  (interactive)
+(defun rustic-cargo-run-rerun (arg)
+  "Run `cargo run' with `rustic-run-arguments'."
+  (interactive "P")
   (let ((default-directory (or rustic-compilation-directory default-directory)))
+    (setq rustic-run-arguments
+          (if arg
+              (read-from-minibuffer "cargo run arguments: "
+                                    rustic-run-arguments nil nil 'rustic-run-history)
+            rustic-run-arguments))
     (rustic-cargo-run-command rustic-run-arguments)))
 
 (defun rustic--get-run-arguments ()
-  "Helper utility for getting arguments related to 'examples' directory."
+  "Helper utility for getting arguments related to `examples' directory."
   (let ((example-name (rustic-cargo-run-get-relative-example-name)))
     (when example-name
       (concat "--example " example-name))))
 
 (defun rustic-cargo-run-get-relative-example-name ()
-  "Run 'cargo run --example' if current buffer within a 'examples' directory."
+  "Run `cargo run --example' if current buffer within a `examples' directory."
   (let* ((buffer-project-root (rustic-buffer-crate))
          (current-filename (if buffer-file-name
                                buffer-file-name
@@ -680,23 +764,23 @@ in your project like `pwd'"
 
 ;;;###autoload
 (defun rustic-cargo-build (&optional arg)
-  "Run 'cargo build' for the current project, allow configuring
+  "Run `cargo build' for the current project, allow configuring
 `rustic-cargo-build-arguments' when prefix argument (C-u) is enabled."
   (interactive "P")
   (when arg
     (setq rustic-cargo-build-arguments
-          (read-string "Cargo build arguments: " (rustic--populate-minibuffer (list rustic-cargo-build-arguments)))))
+          (read-string "Cargo build arguments: " (rustic--populate-minibuffer (list (rustic-cargo-package-argument) rustic-cargo-build-arguments)))))
   (rustic-run-cargo-command `(,(rustic-cargo-bin)
                               ,rustic-cargo-build-exec-command
                               ,@(split-string rustic-cargo-build-arguments))
                             (list :clippy-fix t)))
 
 (defvar rustic-clean-arguments nil
-  "Holds arguments for 'cargo clean', similar to `compilation-arguments`.")
+  "Holds arguments for `cargo clean', similar to `compilation-arguments`.")
 
 ;;;###autoload
 (defun rustic-cargo-clean (&optional arg)
-  "Run 'cargo clean' for the current project.
+  "Run `cargo clean' for the current project.
 
 If ARG is not nil, use value as argument and store it in `rustic-clean-arguments'.
 When calling this function from `rustic-popup-mode', always use the value of
@@ -715,7 +799,7 @@ When calling this function from `rustic-popup-mode', always use the value of
 
 ;;;###autoload
 (defun rustic-cargo-check (&optional arg)
-  "Run 'cargo check' for the current project, allow configuring
+  "Run `cargo check' for the current project, allow configuring
 `rustic-cargo-check-arguments' when prefix argument (C-u) is enabled."
   (interactive "P")
   (when arg
@@ -727,7 +811,7 @@ When calling this function from `rustic-popup-mode', always use the value of
 
 ;;;###autoload
 (defun rustic-cargo-bench ()
-  "Run 'cargo bench' for the current project."
+  "Run `cargo bench' for the current project."
   (interactive)
   (rustic-run-cargo-command (list (rustic-cargo-bin) "bench")))
 
@@ -753,11 +837,11 @@ The documentation is built if necessary."
 ;;; cargo edit
 
 (defvar rustic-cargo-dependencies "*cargo-add-dependencies*"
-  "Buffer that is used for adding missing dependencies with 'cargo add'.")
+  "Buffer that is used for adding missing dependencies with `cargo add'.")
 
 ;;;###autoload
 (defun rustic-cargo-add (&optional arg)
-  "Add crate to Cargo.toml using 'cargo add'.
+  "Add crate to Cargo.toml using `cargo add'.
 If running with prefix command `C-u', read whole command from minibuffer."
   (interactive "P")
   (let* ((base (concat (rustic-cargo-bin) " add "))
@@ -858,7 +942,7 @@ as string."
 
 ;;;###autoload
 (defun rustic-cargo-rm (&optional arg)
-  "Remove crate from Cargo.toml using 'cargo rm'.
+  "Remove crate from Cargo.toml using `cargo rm'.
 If running with prefix command `C-u', read whole command from minibuffer."
   (interactive "P")
   (let* ((command (if arg
@@ -870,7 +954,7 @@ If running with prefix command `C-u', read whole command from minibuffer."
 
 ;;;###autoload
 (defun rustic-cargo-upgrade (&optional arg)
-  "Upgrade dependencies as specified in the local manifest file using 'cargo upgrade'.
+  "Upgrade dependencies as specified in the local manifest file using `cargo upgrade'.
 If running with prefix command `C-u', read whole command from minibuffer."
   (interactive "P")
   (let* ((command (if arg
@@ -912,7 +996,7 @@ command from minibuffer."
   "Buffer name for install buffers.")
 
 (defvar rustic-install-arguments ""
-  "Holds arguments for 'cargo install', similar to `compilation-arguments`.
+  "Holds arguments for `cargo install', similar to `compilation-arguments`.
 Installs that are executed by `rustic-cargo-current-install' will also be
 stored in this variable.")
 
@@ -931,7 +1015,7 @@ stored in this variable.")
 
 ;;;###autoload
 (defun rustic-cargo-install-rerun ()
-  "Run 'cargo install' with `rustic-install-arguments'."
+  "Run `cargo install' with `rustic-install-arguments'."
   (interactive)
   (rustic-compilation-start rustic-install-arguments
                               (list :buffer rustic-install-buffer-name
@@ -940,7 +1024,7 @@ stored in this variable.")
                                     :directory rustic-install-project-dir)))
 ;;;###autoload
 (defun rustic-cargo-install (&optional arg)
-  "Install rust binary using 'cargo install'.
+  "Install rust binary using `cargo install'.
 If running with prefix command `C-u', read whole command from minibuffer."
   (interactive "P")
   (let* ((command (if arg
