@@ -1,7 +1,7 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "rustic" "3.5"
+(define-package "rustic" "20250630.1332"
   "Rust development environment."
-  '((emacs         "26.1")
+  '((emacs         "28.2")
     (rust-mode     "1.0.6")
     (dash          "2.13.0")
     (f             "0.18.2")
@@ -10,9 +10,8 @@
     (project       "0.3.0")
     (s             "1.10.0")
     (spinner       "1.7.3")
-    (xterm-color   "1.6")
-    (flycheck      "34.0"))
+    (xterm-color   "1.6"))
   :url "https://github.com/emacs-rustic/rustic"
-  :commit "d765680373234a6c231acf20c76b07422afcfdf9"
-  :revdesc "3.5-0-gd76568037323"
+  :commit "bfff139f260c386f60d581edef6df1a0d109a131"
+  :revdesc "bfff139f260c"
   :keywords '("languages"))
