@@ -19,7 +19,7 @@
 
 ;;;###autoload
 (defun rustic-rustfix ()
-  "Run 'cargo fix'."
+  "Run `cargo fix'."
   (interactive)
   (let* ((command (list (rustic-cargo-bin) "fix" "--allow-dirty"))
          (err-buf rustic-rustfix-buffer-name)
